@@ -1,6 +1,6 @@
 /*
-spacenavd - a free software replacement driver for 6dof space-mice.
-Copyright (C) 2007-2010 John Tsiombikas <nuclear@member.fsf.org>
+spnavcfg - an interactive GUI configurator for the spacenavd daemon.
+Copyright (C) 2007-2013 John Tsiombikas <nuclear@member.fsf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -21,16 +21,24 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <limits.h>
 
+#define MAX_AXES		64
 #define MAX_BUTTONS		64
+#define MAX_CUSTOM		64
 
 struct cfg {
-	float sensitivity, sens_trans, sens_rot;
-	int dead_threshold;
-	int invert[6];
-	int map_axis[6];
+	float sensitivity, sens_trans[3], sens_rot[3];
+	int dead_threshold[MAX_AXES];
+	int invert[MAX_AXES];
+	int map_axis[MAX_AXES];
 	int map_button[MAX_BUTTONS];
-	int led;
+	int kbmap[MAX_BUTTONS];
+	char *kbmap_str[MAX_BUTTONS];
+	int led, grab_device;
 	char serial_dev[PATH_MAX];
+	int repeat_msec;
+
+	char *devname[MAX_CUSTOM];	/* custom USB device name list */
+	int devid[MAX_CUSTOM][2];	/* custom USB vendor/product id list */
 };
 
 void default_cfg(struct cfg *cfg);
