@@ -1,6 +1,6 @@
 /*
 spacenavd - a free software replacement driver for 6dof space-mice.
-Copyright (C) 2007-2010 John Tsiombikas <nuclear@member.fsf.org>
+Copyright (C) 2007-2012 John Tsiombikas <nuclear@member.fsf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -22,14 +22,21 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <errno.h>
 #include <setjmp.h>
+#ifdef HAVE_ALLOCA_H
 #include <alloca.h>
+#endif
+#ifdef HAVE_MALLOC_H
+#include <malloc.h>
+#endif
 #include <unistd.h>
 #include <pwd.h>
 #include "proto_x11.h"
 #include "client.h"
 #include "spnavd.h"
 #include "xdetect.h"
+#include "kbemu.h"
 
 
 enum cmd_msg {
@@ -78,8 +85,21 @@ int init_x11(void)
 	/* ... also there won't be an XAUTHORITY env var, so set one up */
 	if(!getenv("XAUTHORITY")) {
 		struct passwd *p = getpwuid(getuid());
-		char *home = p->pw_dir ? p->pw_dir : "/tmp";
-		char *buf = alloca(strlen("XAUTHORITY=") + strlen(home) + strlen("/.Xauthority") + 1);
+		char *home, *buf;
+		if(!p || !p->pw_dir) {
+			if(!p) {
+				fprintf(stderr, "getpwuid failed: %s\n", strerror(errno));
+			}
+			fprintf(stderr, "falling back to getting the home directory from the HOME env var...\n");
+			if(!(home = getenv("HOME"))) {
+				fprintf(stderr, "HOME env var not found, using /tmp as a home directory...\n");
+				home = "/tmp";
+			}
+		} else {
+			home = p->pw_dir;
+		}
+
+		buf = alloca(strlen("XAUTHORITY=") + strlen(home) + strlen("/.Xauthority") + 1);
 		sprintf(buf, "XAUTHORITY=%s/.Xauthority", home);
 		putenv(buf);
 	}
@@ -144,6 +164,9 @@ int init_x11(void)
 	}
 	XFlush(dpy);
 
+	/* pass the display connection to the keyboard emulation module */
+	kbemu_set_display(dpy);
+
 	xdet_stop();	/* stop X server detection if it was running */
 	return 0;
 }
@@ -168,6 +191,9 @@ void close_x11(void)
 		XDestroyWindow(dpy, win);
 		XCloseDisplay(dpy);
 		dpy = 0;
+
+		/* stop the kbemu module from using an invalid Display* */
+		kbemu_set_display(0);
 	}
 
 	/* also remove all x11 clients from the client list */
