@@ -1,6 +1,6 @@
 /*
 spacenavd - a free software replacement driver for 6dof space-mice.
-Copyright (C) 2007-2010 John Tsiombikas <nuclear@member.fsf.org>
+Copyright (C) 2007-2013 John Tsiombikas <nuclear@member.fsf.org>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,14 +18,18 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 #include <errno.h>
 #include <signal.h>
 #include <assert.h>
 #include <unistd.h>
 #include <fcntl.h>
 #include <sys/select.h>
+#include <sys/socket.h>
+#include <sys/un.h>
 #include "spnavd.h"
 #include "dev.h"
+#include "hotplug.h"
 #include "client.h"
 #include "proto_unix.h"
 #ifdef USE_X11
@@ -35,13 +39,14 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 static void cleanup(void);
 static void daemonize(void);
 static int write_pid_file(void);
+static int find_running_daemon(void);
 static void handle_events(fd_set *rset);
 static void sig_handler(int s);
 
 
 int main(int argc, char **argv)
 {
-	int i, ret, become_daemon = 1;
+	int i, pid, ret, become_daemon = 1;
 
 	for(i=1; i<argc; i++) {
 		if(argv[i][0] == '-' && argv[i][2] == 0) {
@@ -72,6 +77,11 @@ int main(int argc, char **argv)
 		}
 	}
 
+	if((pid = find_running_daemon()) != -1) {
+		fprintf(stderr, "Spacenav daemon already running (pid: %d). Aborting.\n", pid);
+		return 1;
+	}
+
 	if(become_daemon) {
 		daemonize();
 	}
@@ -81,10 +91,6 @@ int main(int argc, char **argv)
 
 	read_cfg("/etc/spnavrc", &cfg);
 
-	if(init_clients() == -1) {
-		return 1;
-	}
-
 	signal(SIGINT, sig_handler);
 	signal(SIGTERM, sig_handler);
 	signal(SIGSEGV, sig_handler);
@@ -92,9 +98,9 @@ int main(int argc, char **argv)
 	signal(SIGUSR1, sig_handler);
 	signal(SIGUSR2, sig_handler);
 
-	if(init_dev() == -1) {
-		init_hotplug();
-	}
+	init_devices();
+	init_hotplug();
+
 	init_unix();
 #ifdef USE_X11
 	init_x11();
@@ -105,12 +111,21 @@ int main(int argc, char **argv)
 	for(;;) {
 		fd_set rset;
 		int fd, max_fd = 0;
-		struct client *c;
+		struct client *client_iter;
+		struct device *dev;
 
 		FD_ZERO(&rset);
 
-		/* set the device fd if it's open, otherwise set the hotplug fd */
-		if((fd = get_dev_fd()) != -1 || (fd = get_hotplug_fd()) != -1) {
+		dev = get_devices();
+		while(dev) {
+			if((fd = get_device_fd(dev)) != -1) {
+				FD_SET(fd, &rset);
+				if(fd > max_fd) max_fd = fd;
+			}
+			dev = dev->next;
+		}
+
+		if((fd = get_hotplug_fd()) != -1) {
 			FD_SET(fd, &rset);
 			if(fd > max_fd) max_fd = fd;
 		}
@@ -122,16 +137,16 @@ int main(int argc, char **argv)
 		}
 
 		/* all the UNIX socket clients */
-		c = first_client();
-		while(c) {
-			if(get_client_type(c) == CLIENT_UNIX) {
-				int s = get_client_socket(c);
+		client_iter = first_client();
+		while(client_iter) {
+			if(get_client_type(client_iter) == CLIENT_UNIX) {
+				int s = get_client_socket(client_iter);
 				assert(s >= 0);
 
 				FD_SET(s, &rset);
 				if(s > max_fd) max_fd = s;
 			}
-			c = next_client();
+			client_iter = next_client();
 		}
 
 		/* and the X server socket */
@@ -143,11 +158,38 @@ int main(int argc, char **argv)
 #endif
 
 		do {
-			ret = select(max_fd + 1, &rset, 0, 0, 0);
+			/* if there is at least one device out of the deadzone and repeat is enabled
+			 * wait for only as long as specified in cfg.repeat_msec
+			 */
+			struct timeval tv, *timeout = 0;
+			if(cfg.repeat_msec >= 0) {
+				dev = get_devices();
+				while(dev) {
+					if(is_device_valid(dev) && !in_deadzone(dev)) {
+						tv.tv_sec = cfg.repeat_msec / 1000;
+						tv.tv_usec = cfg.repeat_msec % 1000;
+						timeout = &tv;
+						break;
+					}
+					dev = dev->next;
+				}
+			}
+
+			ret = select(max_fd + 1, &rset, 0, 0, timeout);
 		} while(ret == -1 && errno == EINTR);
 
 		if(ret > 0) {
 			handle_events(&rset);
+		} else {
+			if(cfg.repeat_msec >= 0) {
+				dev = get_devices();
+				while(dev) {
+					if(!in_deadzone(dev)) {
+						repeat_last_event(dev);
+					}
+					dev = dev->next;
+				}
+			}
 		}
 	}
 	return 0;	/* unreachable */
@@ -155,11 +197,22 @@ int main(int argc, char **argv)
 
 static void cleanup(void)
 {
+	struct device *dev;
+
 #ifdef USE_X11
 	close_x11();	/* call to avoid leaving garbage in the X server's root windows */
 #endif
 	close_unix();
-	shutdown_dev();
+
+	shutdown_hotplug();
+
+	dev = get_devices();
+	while(dev) {
+		struct device *tmp = dev;
+		dev = dev->next;
+		remove_device(tmp);
+	}
+
 	remove(PIDFILE);
 }
 
@@ -205,9 +258,45 @@ static int write_pid_file(void)
 	return 0;
 }
 
+static int find_running_daemon(void)
+{
+	FILE *fp;
+	int s, pid;
+	struct sockaddr_un addr;
+
+	/* try to open the pid-file */
+	if(!(fp = fopen(PIDFILE, "r"))) {
+		return -1;
+	}
+	if(fscanf(fp, "%d\n", &pid) != 1) {
+		fclose(fp);
+		return -1;
+	}
+	fclose(fp);
+
+	/* make sure it's not just a stale pid-file */
+	if((s = socket(PF_UNIX, SOCK_STREAM, 0)) == -1) {
+		return -1;
+	}
+	memset(&addr, 0, sizeof addr);
+	addr.sun_family = AF_UNIX;
+	strncpy(addr.sun_path, SOCK_NAME, sizeof addr.sun_path);
+
+	if(connect(s, (struct sockaddr*)&addr, sizeof addr) == -1) {
+		close(s);
+		return -1;
+	}
+
+	/* managed to connect alright, it's running... */
+	close(s);
+	return pid;
+}
+
 static void handle_events(fd_set *rset)
 {
 	int dev_fd, hotplug_fd;
+	struct device *dev;
+	struct dev_input inp;
 
 	/* handle anything coming through the UNIX socket */
 	handle_uevents(rset);
@@ -218,18 +307,24 @@ static void handle_events(fd_set *rset)
 #endif
 
 	/* finally read any pending device input data */
-	if((dev_fd = get_dev_fd()) != -1) {
-		if(FD_ISSET(dev_fd, rset)) {
-			struct dev_input inp;
+	dev = get_devices();
+	while(dev) {
+		/* keep the next pointer because read_device can potentially destroy
+		 * the device node if the read fails.
+		 */
+		struct device *next = dev->next;
 
+		if((dev_fd = get_device_fd(dev)) != -1 && FD_ISSET(dev_fd, rset)) {
 			/* read an event from the device ... */
-			while(read_dev(&inp) != -1) {
+			while(read_device(dev, &inp) != -1) {
 				/* ... and process it, possibly dispatching a spacenav event to clients */
-				process_input(&inp);
+				process_input(dev, &inp);
 			}
 		}
+		dev = next;
+	}
 
-	} else if((hotplug_fd = get_hotplug_fd()) != -1) {
+	if((hotplug_fd = get_hotplug_fd()) != -1) {
 		if(FD_ISSET(hotplug_fd, rset)) {
 			handle_hotplug();
 		}
@@ -241,14 +336,22 @@ static void handle_events(fd_set *rset)
  */
 static void sig_handler(int s)
 {
-	int tmp;
+	int prev_led = cfg.led;
 
 	switch(s) {
 	case SIGHUP:
-		tmp = cfg.led;
 		read_cfg("/etc/spnavrc", &cfg);
-		if(cfg.led != tmp) {
-			set_led(cfg.led);
+		if(cfg.led != prev_led) {
+			struct device *dev = get_devices();
+			while(dev) {
+				if(is_device_valid(dev)) {
+					if(verbose) {
+						printf("turn led %s, device: %s\n", cfg.led ? "on": "off", dev->name);
+					}
+					set_device_led(dev, cfg.led);
+				}
+				dev = dev->next;
+			}
 		}
 		break;
 
