@@ -25,6 +25,24 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #define SOCK_NAME	"/var/run/spnav.sock"
 #define PIDFILE		"/var/run/spnavd.pid"
 #define LOGFILE		"/var/log/spnavd.log"
+/* Multiple devices support */
+#ifndef MAX_DEVICES
+#define MAX_DEVICES 8
+#endif
+
+#if defined(__cplusplus) || (__STDC_VERSION__ >= 199901L)
+#define INLINE	inline
+#else	/* not C++ or C99 */
+
+#ifdef __GNUC__
+#define INLINE	__inline__
+#else
+#define INLINE
+#endif
+
+#endif
+
+
 
 struct cfg cfg;
 int verbose;
